searchData={"items":[{"type":"module","title":"Goth","doc":"A Goth token server.","ref":"Goth.html"},{"type":"function","title":"Goth.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Goth.html#child_spec/1"},{"type":"function","title":"Goth.fetch/2","doc":"Fetches the token from the cache.\n\nIf the token is not in the cache, this function blocks for `timeout`\nmilliseconds (defaults to `5000`) while it is attempted to fetch\nit in the background.\n\nTo fetch the token bypassing the cache, see `Goth.Token.fetch/1`.","ref":"Goth.html#fetch/2"},{"type":"function","title":"Goth.fetch!/2","doc":"Fetches the token, erroring if it is missing.\n\nSee `fetch/2` for more information.","ref":"Goth.html#fetch!/2"},{"type":"function","title":"Goth.start_link/1","doc":"Starts the server.\n\nWhen the server is started, we attempt to fetch the token and store it in\ninternal cache. If we fail, we'll retry with backoff.","ref":"Goth.html#start_link/1"},{"type":"function","title":"Options - Goth.start_link/1","doc":"* `:name` - a unique name to register the server under. It can be any term.\n\n  * `:source` - the source to retrieve the token from.\n\n    Supported values include:\n\n      * `{:service_account, credentials}` - fetch token using service account credentials\n\n      * `{:refresh_token, credentials}` - fetch token using refresh token\n\n      * `:metadata` - fetching token using Google internal metadata service\n\n    If `:source` is not set, Goth will:\n\n      * Check application environment. You can set it with: `config :goth, json: File.read!(\"credentials.json\")`.\n\n      * Check `GOOGLE_APPLICATION_CREDENTIALS` env variable that contains path to credentials file.\n\n      * Check `GOOGLE_APPLICATION_CREDENTIALS_JSON` env variable that contains credentials JSON.\n\n      * Check `~/.config/gcloud/application_default_credentials.json` file.\n\n      * Check `~/.config/gcloud/configurations/config_default` for project id and email address for SDK users.\n\n      * Check Google internal metadata service\n\n      * Otherwise, raise an error.\n\n    See documentation of the \"Source\" section in `Goth.Token.fetch/1` documentation\n    for more information.\n\n  * `:refresh_before` - Time in seconds before the token is about to expire\n    that it is tried to be automatically refreshed. Defaults to\n    `300` (5 minutes).\n\n  * `:http_client` - a function that makes the HTTP request. Defaults to using built-in\n    integration with [Finch](https://github.com/sneako/finch)\n\n    See documentation of the `:http_client` option in `Goth.Token.fetch/1` for\n    more information.\n\n  * `:prefetch` - how to prefetch the token when the server starts. The possible options\n    are `:async` to do it asynchronously or `:sync` to do it synchronously\n    (that is, the server doesn't start until an attempt to fetch the token was made). Defaults\n    to `:async`.\n\n  * `:max_retries` - the maximum number of retries (default: `10`)\n\n  * `:retry_delay` - a function that receives the retry count (starting at 0) and returns the delay, the\n    number of milliseconds to sleep before making another attempt.\n    Defaults to a simple exponential backoff capped at 30s: 1s, 2s, 4s, 8s, 16s, 30s, 30s, ...","ref":"Goth.html#start_link/1-options"},{"type":"function","title":"Examples - Goth.start_link/1","doc":"Generate a token using a service account credentials file:\n\n    iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    iex> {:ok, _} = Goth.start_link(name: MyApp.Goth, source: {:service_account, credentials, []})\n    iex> Goth.fetch!(MyApp.Goth)\n    %Goth.Token{...}\n\nRetrieve the token using a refresh token:\n\n    iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    iex> {:ok, _} = Goth.start_link(name: MyApp.Goth, source: {:refresh_token, credentials, []})\n    iex> Goth.fetch!(MyApp.Goth)\n    %Goth.Token{...}\n\nRetrieve the token using the Google metadata server:\n\n    iex> {:ok, _} = Goth.start_link(name: MyApp.Goth, source: {:metadata, []})\n    iex> Goth.fetch!(MyApp.Goth)\n    %Goth.Token{...}","ref":"Goth.html#start_link/1-examples"},{"type":"module","title":"Goth.Token","doc":"Functions for retrieving the token from the Google API.","ref":"Goth.Token.html"},{"type":"function","title":"Goth.Token.fetch/1","doc":"Fetch the token from the Google API using the given `config`.\n\nConfig may contain the following keys:\n\n  * `:source` - the source to retrieve the token from.\n\n    Supported values include:\n\n      * `{:service_account, credentials}` - for fetching token using service account credentials\n\n      * `{:refresh_token, credentials}` - for fetching token using refresh token\n\n      * `:metadata` - for fetching token using Google internal metadata service\n\n    If `:source` is not set, Goth will:\n\n      * Check application environment. You can set it with: `config :goth, json: File.read!(\"credentials.json\")`.\n\n      * Check `GOOGLE_APPLICATION_CREDENTIALS` env variable that contains path to credentials file.\n\n      * Check `GOOGLE_APPLICATION_CREDENTIALS_JSON` env variable that contains credentials JSON.\n\n      * Check `~/.config/gcloud/application_default_credentials.json` file.\n\n      * Check Google internal metadata service\n\n      * Otherwise, raise an error.\n\n    See \"Source\" section below for more information.\n\n  * `:http_client` - a function that makes the HTTP request.\n\n    Can be one of the following:\n\n      * `fun` - same as `{fun, []}`\n\n      * `{fun, opts}` - `fun` must be a 1-arity function that receives a keyword list with fields\n        `:method`, `:url`, `:headers`, and `:body` along with any passed `opts`. The function must return\n        `{:ok, %{status: status, headers: headers, body: body}}` or `{:error, exception}`.\n\n        See \"Custom HTTP Client\" section below for more information.\n\n    `fun` can also be an atom `:finch` to use the built-in [Finch](http://github.com/sneako/finch)-based\n    client.\n\n    Defaults to `{:finch, []}`.","ref":"Goth.Token.html#fetch/1"},{"type":"function","title":"Source - Goth.Token.fetch/1","doc":"Source can be one of:\n\n##","ref":"Goth.Token.html#fetch/1-source"},{"type":"function","title":"Service account - `{:service_account, credentials}` - Goth.Token.fetch/1","doc":"Same as `{:service_account, credentials, []}`\n\n##","ref":"Goth.Token.html#fetch/1-service-account-service_account-credentials"},{"type":"function","title":"Service account - `{:service_account, credentials, options}` - Goth.Token.fetch/1","doc":"The `credentials` is a map and can contain the following keys:\n\n  * `\"private_key\"`\n\n  * `\"client_email\"`\n\nThe `options` is a keywords list and can contain the following keys:\n\n  * `:url` - the URL of the authentication service, defaults to:\n    `\"https://www.googleapis.com/oauth2/v4/token\"`\n\n  * `:scopes` - the list of token scopes, defaults to `[\"https://www.googleapis.com/auth/cloud-platform\"]` (ignored if `:claims` present)\n\n  * `:claims` - self-signed JWT extra claims. Should be a map with string keys only.\n    A self-signed JWT will be [exchanged for a Google-signed ID token](https://cloud.google.com/functions/docs/securing/authenticating#exchanging_a_self-signed_jwt_for_a_google-signed_id_token)\n\n##","ref":"Goth.Token.html#fetch/1-service-account-service_account-credentials-options"},{"type":"function","title":"Refresh token - `{:refresh_token, credentials}` - Goth.Token.fetch/1","doc":"Same as `{:refresh_token, credentials, []}`\n\n##","ref":"Goth.Token.html#fetch/1-refresh-token-refresh_token-credentials"},{"type":"function","title":"Refresh token - `{:refresh_token, credentials, options}` - Goth.Token.fetch/1","doc":"The `credentials` is a map and can contain the following keys:\n\n  * `\"refresh_token\"`\n\n  * `\"client_id\"`\n\n  * `\"client_secret\"`\n\nThe `options` is a keywords list and can contain the following keys:\n\n  * `:url` - the URL of the authentication service, defaults to:\n    `\"https://www.googleapis.com/oauth2/v4/token\"`\n\n##","ref":"Goth.Token.html#fetch/1-refresh-token-refresh_token-credentials-options"},{"type":"function","title":"Google metadata server - `:metadata` - Goth.Token.fetch/1","doc":"Same as `{:metadata, []}`\n\n##","ref":"Goth.Token.html#fetch/1-google-metadata-server-metadata"},{"type":"function","title":"Google metadata server - `{:metadata, options}` - Goth.Token.fetch/1","doc":"The `options` is a keywords list and can contain the following keys:\n\n  * `:account` - the name of the account to generate the token for, defaults to `\"default\"`\n\n  * `:url` - the URL of the metadata server, defaults to `\"http://metadata.google.internal\"`\n\n  * `:audience` - the audience you want an identity token for, default to `nil`\n    If this parameter is provided, an identity token is returned instead of an access token","ref":"Goth.Token.html#fetch/1-google-metadata-server-metadata-options"},{"type":"function","title":"Custom HTTP Client - Goth.Token.fetch/1","doc":"To use a custom HTTP client, define a function that receives a keyword list with fields\n`:method`, `:url`, `:headers`, and `:body`. The function must return\n`{:ok, %{status: status, headers: headers, body: body}}` or `{:error, exception}`.\n\nHere's an example with Finch:\n\n    defmodule MyApp do\n      def request_with_finch(options) do\n        {method, options} = Keyword.pop!(options, :method)\n        {url, options} = Keyword.pop!(options, :url)\n        {headers, options} = Keyword.pop!(options, :headers)\n        {body, options} = Keyword.pop!(options, :body)\n\n        Finch.build(method, url, headers, body)\n        |> Finch.request(Goth.Finch, options)\n      end\n    end\n\nAnd here is how it can be used:\n\n    iex> Goth.Token.fetch(source: source, http_client: &MyApp.request_with_finch/1)\n    {:ok, %Goth.Token{...}}\n\n    iex> Goth.Token.fetch(source: source, http_client: {&MyApp.request_with_finch/1, receive_timeout: 5000})\n    {:ok, %Goth.Token{...}}","ref":"Goth.Token.html#fetch/1-custom-http-client"},{"type":"function","title":"Examples - Goth.Token.fetch/1","doc":"##","ref":"Goth.Token.html#fetch/1-examples"},{"type":"function","title":"Generate a token using a service account credentials file: - Goth.Token.fetch/1","doc":"iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    iex> Goth.Token.fetch(source: {:service_account, credentials})\n    {:ok, %Goth.Token{...}}\n\nYou can generate a credentials file containing service account using `gcloud` utility like this:\n\n    $ gcloud iam service-accounts keys create --key-file-type=json --iam-account=... credentials.json\n\n##","ref":"Goth.Token.html#fetch/1-generate-a-token-using-a-service-account-credentials-file"},{"type":"function","title":"Generate a cloud function invocation token using a service account credentials file: - Goth.Token.fetch/1","doc":"iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    ...> claims = %{\"target_audience\" => \"https:// - .cloudfunctions.net/ \"}\n    ...> Goth.Token.fetch(source: {:service_account, credentials, [claims: claims]})\n    {:ok, %Goth.Token{...}}\n\n##","ref":"Goth.Token.html#fetch/1-generate-a-cloud-function-invocation-token-using-a-service-account-credentials-file"},{"type":"function","title":"Generate an impersonated token using a service account credentials file: - Goth.Token.fetch/1","doc":"iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    ...> claims = %{\"sub\" => \" \"}\n    ...> Goth.Token.fetch(source: {:service_account, credentials, [claims: claims]})\n    {:ok, %Goth.Token{...}}\n\n##","ref":"Goth.Token.html#fetch/1-generate-an-impersonated-token-using-a-service-account-credentials-file"},{"type":"function","title":"Retrieve the token using a refresh token: - Goth.Token.fetch/1","doc":"iex> credentials = \"credentials.json\" |> File.read!() |> Jason.decode!()\n    iex> Goth.Token.fetch(source: {:refresh_token, credentials})\n    {:ok, %Goth.Token{...}}\n\nYou can generate a credentials file containing refresh token using `gcloud` utility like this:\n\n    $ gcloud auth application-default login\n\n##","ref":"Goth.Token.html#fetch/1-retrieve-the-token-using-a-refresh-token"},{"type":"function","title":"Retrieve the token using the Google metadata server: - Goth.Token.fetch/1","doc":"iex> Goth.Token.fetch(source: :metadata)\n    {:ok, %Goth.Token{...}}\n\nSee [Storing and retrieving instance metadata](https://cloud.google.com/compute/docs/storing-retrieving-metadata)\nfor more information on metadata server.\n\n\n##","ref":"Goth.Token.html#fetch/1-retrieve-the-token-using-the-google-metadata-server"},{"type":"function","title":"Passing custom Finch options - Goth.Token.fetch/1","doc":"iex> Goth.Token.fetch(source: source, http_client: {:finch, pool_timeout: 5000})\n    {:ok, %Goth.Token{...}}","ref":"Goth.Token.html#fetch/1-passing-custom-finch-options"},{"type":"function","title":"Goth.Token.for_scope/2","doc":"","ref":"Goth.Token.html#for_scope/2"},{"type":"type","title":"Goth.Token.t/0","doc":"","ref":"Goth.Token.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.4.3 (2024-01-15) - Changelog","doc":"* Update Finch to 0.17","ref":"changelog.html#v1-4-3-2024-01-15"},{"type":"extras","title":"v1.4.2 (2023-08-24) - Changelog","doc":"* Update jose dependency\n  * Require Elixir 1.11+","ref":"changelog.html#v1-4-2-2023-08-24"},{"type":"extras","title":"v1.4.1 (2023-06-27) - Changelog","doc":"* change to OS time to prevent unmanagable clock skew","ref":"changelog.html#v1-4-1-2023-06-27"},{"type":"extras","title":"v1.4.0 (2023-03-23) - Changelog","doc":"* Load config from `~/.config/gcloud/configurations/config_default`","ref":"changelog.html#v1-4-0-2023-03-23"},{"type":"extras","title":"v1.3.1 (2022-08-10) - Changelog","doc":"* Force refresh whenever cached token is expired\n  * Fix getting tokens from metadata with no options provide","ref":"changelog.html#v1-3-1-2022-08-10"},{"type":"extras","title":"v1.3.0 (2022-06-29) - Changelog","doc":"* Add `Goth.start_link/1`, `Goth.fetch/2`, `Goth.fetch!/2`, and `Goth.Token.fetch/1`.\n\nDeprecations:\n\n  * `Goth.Token`: Deprecate `for_scope/2`, `from_response_json/2`, `refresh!/1`\n  * Deprecate `Goth.Client` module\n  * Deprecate `Goth.Config` module\n  * Deprecate `Goth.TokenStore` module","ref":"changelog.html#v1-3-0-2022-06-29"},{"type":"extras","title":"Goth","doc":"# Goth\n\n![CI](https://github.com/peburrows/goth/workflows/CI/badge.svg)\n\n\nGoogle + Auth = Goth\n\nA simple library to generate and retrieve OAuth2 tokens for use with Google\nCloud Service accounts.","ref":"readme.html"},{"type":"extras","title":"Installation - Goth","doc":"**Note:** below are instructions for using Goth v1.3+. For more information on\nearlier versions of Goth, [see v1.2.0 documentation on hexdocs.pm](https://hexdocs.pm/goth/1.2.0).\n\n1. Add `:goth` to your list of dependencies in `mix.exs`.\n\n   ```elixir\n   def deps do\n     [\n       {:goth, \"~> 1.3\"}\n     ]\n   end\n   ```\n\n2. Add Goth to your supervision tree:\n\n   ```elixir\n   defmodule MyApp.Application do\n     use Application\n\n     def start(_type, _args) do\n       credentials =\n         \"GOOGLE_APPLICATION_CREDENTIALS_JSON\"\n         |> System.fetch_env!()\n         |> Jason.decode!()\n\n       source = {:service_account, credentials}\n\n       children = [\n         {Goth, name: MyApp.Goth, source: source}\n       ]\n\n       Supervisor.start_link(children, strategy: :one_for_one)\n     end\n   end\n   ```\n\n   If you set `GOOGLE_APPLICATION_CREDENTIALS` or\n   `GOOGLE_APPLICATION_CREDENTIALS_JSON`, have a\n   `~/.config/gcloud/application_default_credentials.json` file,\n   `~/.config/gcloud/configurations/config_default` file or deploy\n   your application to Google Cloud, you can omit the `:source` option:\n\n   ```elixir\n   def start(_type, _args) do\n     children = [\n       {Goth, name: MyApp.Goth}\n     ]\n\n     Supervisor.start_link(children, strategy: :one_for_one)\n   end\n   ```\n\n   If you want to use multiple credentials, you may consider doing:\n\n   ```elixir\n   def start(_type, _args) do\n     Supervisor.start_link(servers(), strategy: :one_for_one)\n   end\n\n   defp servers do\n     servers = [\n       {MyApp.Cred1, source1},\n       ...\n       {MyApp.CredN, source2}\n     ]\n\n     for {name, source} <- servers do\n       Supervisor.child_spec({Goth, name: name, source: source}, id: name)\n     end\n   end\n   ```\n\n3. Fetch the token:\n\n   ```elixir\n   iex> Goth.fetch!(MyApp.Goth)\n   %Goth.Token{\n     expires: 1453356568,\n     token: \"ya29.cALlJ4ICWRvMkYB-WsAR-CZnExE459PA7QPqKg5nei9y2T9-iqmbcgxq8XrTATNn_BPim\",\n     type: \"Bearer\",\n     ...\n   }\n   ```\n\nSee `Goth.start_link/1` for more information about possible configuration options.","ref":"readme.html#installation"},{"type":"extras","title":"Upgrading from Goth 1.2 - Goth","doc":"See [Upgrading from Goth 1.2](UPGRADE_GUIDE.md) guide for more information.","ref":"readme.html#upgrading-from-goth-1-2"},{"type":"extras","title":"Community resources - Goth","doc":"- [How to upload on YouTube Data API with elixir ?](https://mrdotb.com/posts/upload-on-youtube-with-elixir/)","ref":"readme.html#community-resources"},{"type":"extras","title":"Copyright and License - Goth","doc":"Copyright (c) 2016 Phil Burrows\n\nThis work is free. You can redistribute it and/or modify it under the terms of\nthe MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Upgrading from Goth 1.2","doc":"# Upgrading from Goth 1.2\n\nEarlier versions of Goth relied on global application environment configuration\nwhich is deprecated in favour of a more direct and explicit approach in Goth\nv1.3+.\n\nBelow is a step-by-step upgrade path from Goth 1.x to 1.3:\n\nChange your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:goth, \"~> 1.3-rc\"}\n  ]\nend\n```\n\nYou might have code similar to this:\n\n```elixir\n# config/config.exs\nconfig :goth,\n  json: {:system, \"GCP_CREDENTIALS\"}\n```\n\n```elixir\n# lib/myapp.ex\ndefmodule MyApp do\n  def gcloud_authorization() do\n    {:ok, token} = Goth.Token.for_scope(\"https://www.googleapis.com/auth/cloud-platform.read-only\")\n    \"#{token.type} #{token.token}\"\n  end\nend\n```\n\nReplace it with:\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n  use Application\n\n  def start(_type, _args) do\n    credentials = \"GCP_CREDENTIALS\" |> System.fetch_env!() |> Jason.decode!()\n    scopes = [\"https://www.googleapis.com/auth/cloud-platform.read-only\"]\n    source = {:service_account, credentials, scopes: scopes}\n\n    children = [\n      {Goth, name: MyApp.Goth, source: source}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\n```elixir\n# lib/myapp.ex\ndefmodule MyApp do\n  def gcloud_authorization() do\n    {:ok, token} = Goth.fetch(MyApp.Goth)\n    \"#{token.type} #{token.token}\"\n  end\nend\n```\n\nFor more information on earlier versions of Goth, [see v1.2.0 documentation on hexdocs.pm](https://hexdocs.pm/goth/1.2.0).","ref":"upgrade_guide.html"},{"type":"extras","title":"The MIT License (MIT)","doc":"# The MIT License (MIT)\n\nCopyright (c) 2016 Phil Burrows\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}